/*
 * file ViewHelper.java
 *
 * Licensed Materials - Property of IBM
 * Restricted Materials of IBM - you are allowed to copy, modify and 
 * redistribute this file as part of any program that interfaces with 
 * IBM Rational CM API.
 *
 * com.ibm.rational.stp.client.samples.CheckoutDemo
 *
 * (C) Copyright IBM Corporation 2008, 2010.  All Rights Reserved.
 * Note to U.S. Government Users Restricted Rights:  Use, duplication or 
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */
package com.ibm.rational.cmapi.samples.cc;

import java.io.File;
import java.util.Date;
import java.util.Random;

import com.ibm.rational.wvcm.stp.StpLocation;
import com.ibm.rational.wvcm.stp.StpProvider.Domain;
import com.ibm.rational.wvcm.stp.cc.CcDirectory;
import com.ibm.rational.wvcm.stp.cc.CcFile;
import com.ibm.rational.wvcm.stp.cc.CcProvider;
import com.ibm.rational.wvcm.stp.cc.CcView;

public class ViewHelper {

    private CcView m_view;
    private final CcProvider m_provider;
    private CcDirectory m_demoDir;
    private CcDirectory m_vobRootDir;
    private final CcDemoProps m_props;

    public static ViewHelper getNonUcmViewHelper(CcDemoBase env) {
        return new ViewHelper(env.getClearCaseProvider(), env.props());
    }

    public ViewHelper(CcProvider provider, CcDemoProps props) {
        m_provider = provider;
        m_props = props;
    }

    void tearDown() throws Exception {
        // Delete demo web view
        if (m_view != null) {
            m_view.doUnbindAll(null);
            m_view = null;
        }
    }

    /**
     * Create a temporary ClearCase web view.
     * This view will be deleted once the demo completes.
     * @throws Exception 
     */
    protected CcView getView() throws Exception {
        if (m_view == null) {
            // Create a location for the web view rooted in the user's temp dir.
            // This does not actually create the view itself.
            File tempDir = m_props.getTempDir();
            String viewTag = generateUniqueName(m_props.getLoginName() + ".demo.");
            File viewRoot = new File(tempDir, viewTag);
            StpLocation viewLoc = m_provider.filePathLocation(Domain.CLEAR_CASE, viewRoot);

            // Create the view itself
            m_view = m_provider.ccView(viewLoc).doCreateCcWebView(null);
        }
        return m_view;
    }

    /**
     * Generate a (hopefully) unique name by adding a random integer to the
     * specified base name.
     */
    static String generateUniqueName(String baseName) {
        Random random = new Random(new Date().getTime());
        return baseName + random.nextInt(100000);
    }

    CcFile createDemoFile(String fileName, boolean controlled) throws Exception {
        CcDirectory parentDir = createDemoDirectory();
        StpLocation demoFileLoc = (StpLocation) parentDir.stpLocation().child(fileName);
        CcFile demoFile = m_provider.ccFile(demoFileLoc);

        // Create the file in the web view's local file area.
        demoFile = demoFile.createCcFile(null);

        if (controlled) {
            // Version control the demo file.
            demoFile = (CcFile) demoFile.doVersionControl(null);
        }
        return demoFile;
    }

    /**
     * Create a version-controlled directory in the demo VOB.
     */
    private CcDirectory createDemoDirectory() throws Exception {
        if (m_demoDir == null) {
            // Get the demo VOB's root directory.
            // That's where we're going to create the demo directory.
            CcDirectory vobRootDir = getDemoVobRootDirectory();

            // Create a location object for the demo directory.
            // The demo directory will be a child of the demo VOB's root directory.
            String demoDirName = generateUniqueName("demo");
            StpLocation vobRootDirLoc = vobRootDir.stpLocation();
            StpLocation demoDirLoc = (StpLocation) vobRootDirLoc.child(demoDirName);

            // Create a proxy object for the demo directory.
            CcDirectory demoDir = m_provider.ccDirectory(demoDirLoc);

            // Create the directory in the web view's local file area.
            demoDir = demoDir.createCcDirectory(null);

            // Version control the demo directory.
            m_demoDir = (CcDirectory) demoDir.doVersionControl(null);
        }
        return m_demoDir;
    }

    /**
     * Get the root directory of the demo VOB in the context of the
     * demo web view.
     */
    private CcDirectory getDemoVobRootDirectory() throws Exception {
        if (m_vobRootDir == null) {
            // Get demo VOB's tag.  Strip leading slash.
            String demoVobTag = m_props.getDemoUcmProjectVob().substring(1);

            // The demo VOB's root directory is a child of the web view.
            CcView view = getView();

            // Create a location object for the VOB root directory.
            StpLocation viewLoc = view.stpLocation();
            StpLocation vobRootDirLoc = (StpLocation) viewLoc.child(demoVobTag);

            // Create a proxy object for the VOB root directory.
            m_vobRootDir = m_provider.ccDirectory(vobRootDirLoc);

            // Load the demo VOB into the demo web view.
            m_vobRootDir.doLoad(null);
        }
        return m_vobRootDir;
    }
}
